'use strict';

var isProduction = process.env.NODE_ENV === 'production';

module.exports = isProduction ? {} : plugin;

function plugin(_ref) {
  var t = _ref.types;

  var findBareSupers = {
    Super(path) {
      if (path.parentPath.isCallExpression({ callee: path.node })) {
        this.push(path.parentPath);
      }
    }
  };

  var buildClassMethod = function buildClassMethod(node) {
    var key = node.key,
        value = node.value;


    var body = void 0;

    if (value.body.type === 'BlockStatement') {
      body = value.body;
    } else {
      body = t.blockStatement([t.returnStatement(value.body)]);
    }

    var method = t.classMethod('method', t.identifier(key.name), value.params, body);

    method.async = value.async;

    return method;
  };

  var buildBindStatement = function buildBindStatement(ref, _ref2) {
    var key = _ref2.key,
        computed = _ref2.computed;

    var member = t.memberExpression(ref, key, computed || t.isLiteral(key));

    var value = t.callExpression(t.memberExpression(member, t.identifier('bind')), [t.thisExpression()]);

    return t.expressionStatement(t.assignmentExpression('=', member, value));
  };

  var isBoundProperty = function isBoundProperty(path) {
    return path.isClassProperty() && path.node.value && path.node.value.type === 'ArrowFunctionExpression';
  };

  return {
    inherits: require('babel-plugin-syntax-class-properties'),

    visitor: {
      Class(path, state) {
        var props = [];
        var body = path.get('body');
        var isDerived = !!path.node.superClass;

        var constructor = void 0;

        for (var _iterator = body.get('body'), _isArray = Array.isArray(_iterator), _i = 0, _iterator = _isArray ? _iterator : _iterator[Symbol.iterator]();;) {
          var _ref3;

          if (_isArray) {
            if (_i >= _iterator.length) break;
            _ref3 = _iterator[_i++];
          } else {
            _i = _iterator.next();
            if (_i.done) break;
            _ref3 = _i.value;
          }

          var _path = _ref3;

          if (_path.isClassMethod({ kind: 'constructor' })) {
            constructor = _path;
          } else if (isBoundProperty(_path)) {
            props.push(_path);
          }
        }

        if (!props.length) return;

        var bindStatements = [];

        for (var _iterator2 = props, _isArray2 = Array.isArray(_iterator2), _i2 = 0, _iterator2 = _isArray2 ? _iterator2 : _iterator2[Symbol.iterator]();;) {
          var _ref4;

          if (_isArray2) {
            if (_i2 >= _iterator2.length) break;
            _ref4 = _iterator2[_i2++];
          } else {
            _i2 = _iterator2.next();
            if (_i2.done) break;
            _ref4 = _i2.value;
          }

          var prop = _ref4;

          var node = prop.node;

          var method = buildClassMethod(node);

          body.pushContainer('body', [method]);

          bindStatements.push(buildBindStatement(t.thisExpression(), node));
        }

        for (var _iterator3 = props, _isArray3 = Array.isArray(_iterator3), _i3 = 0, _iterator3 = _isArray3 ? _iterator3 : _iterator3[Symbol.iterator]();;) {
          var _ref5;

          if (_isArray3) {
            if (_i3 >= _iterator3.length) break;
            _ref5 = _iterator3[_i3++];
          } else {
            _i3 = _iterator3.next();
            if (_i3.done) break;
            _ref5 = _i3.value;
          }

          var _prop = _ref5;

          _prop.remove();
        }

        if (!constructor) {
          var newConstructor = t.classMethod('constructor', t.identifier('constructor'), [], t.blockStatement([]));

          if (isDerived) {
            newConstructor.params = [t.restElement(t.identifier('args'))];

            newConstructor.body.body.push(t.returnStatement(t.callExpression(t.super(), [t.spreadElement(t.identifier('args'))])));
          }

          var _body$unshiftContaine = body.unshiftContainer('body', newConstructor);

          constructor = _body$unshiftContaine[0];
        }

        if (isDerived) {
          var bareSupers = [];

          constructor.traverse(findBareSupers, bareSupers);

          for (var _iterator4 = bareSupers, _isArray4 = Array.isArray(_iterator4), _i4 = 0, _iterator4 = _isArray4 ? _iterator4 : _iterator4[Symbol.iterator]();;) {
            var _ref6;

            if (_isArray4) {
              if (_i4 >= _iterator4.length) break;
              _ref6 = _iterator4[_i4++];
            } else {
              _i4 = _iterator4.next();
              if (_i4.done) break;
              _ref6 = _i4.value;
            }

            var bareSuper = _ref6;

            bareSuper.insertAfter(bindStatements);
          }
        } else {
          constructor.get('body').unshiftContainer('body', bindStatements);
        }
      }
    }
  };
}